@@ -15,6 +15,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 """
 Created on Dec 30, 2015
 
@@ -28,13 +29,14 @@ import logging
 import os
 import unittest
 
+from rdiffweb.core.store import ADMIN_ROLE, MAINTAINER_ROLE, USER_ROLE
 from rdiffweb.test import WebCase
 
 
 class AbstractAdminTest(WebCase):
     """Class to regroup command method to test admin page."""
 
-    def _add_user(self, username=None, email=None, password=None, user_root=None, is_admin=None):
+    def _add_user(self, username=None, email=None, password=None, user_root=None, role=None):
         b = {}
         b['action'] = 'add'
         if username is not None:
@@ -45,11 +47,11 @@ class AbstractAdminTest(WebCase):
             b['password'] = password
         if user_root is not None:
             b['user_root'] = user_root
-        if is_admin is not None:
-            b['is_admin'] = str(bool(is_admin)).lower()
+        if role is not None:
+            b['role'] = str(role)
         self.getPage("/admin/users/", method='POST', body=b)
 
-    def _edit_user(self, username=None, email=None, password=None, user_root=None, is_admin=None):
+    def _edit_user(self, username=None, email=None, password=None, user_root=None, role=None):
         b = {}
         b['action'] = 'edit'
         if username is not None:
@@ -60,8 +62,8 @@ class AbstractAdminTest(WebCase):
             b['password'] = password
         if user_root is not None:
             b['user_root'] = user_root
-        if is_admin is not None:
-            b['is_admin'] = str(bool(is_admin)).lower()
+        if role is not None:
+            b['role'] = str(role)
         self.getPage("/admin/users/", method='POST', body=b)
 
     def _delete_user(self, username='test1'):
@@ -75,49 +77,68 @@ class AdminUsersAsAdminTest(AbstractAdminTest):
 
     login = True
 
+    def test_add_user_with_role(self):
+        #  Add user to be listed
+        self._add_user("admin_role", "admin_role@test.com", "test2", "/home/", ADMIN_ROLE)
+        self.assertEqual(ADMIN_ROLE, self.app.store.get_user('admin_role').role)
+
+        self._add_user("maintainer_role", "maintainer_role@test.com", "test2", "/home/", MAINTAINER_ROLE)
+        self.assertEqual(MAINTAINER_ROLE, self.app.store.get_user('maintainer_role').role)
+
+        self._add_user("user_role", "user_role@test.com", "test2", "/home/", USER_ROLE)
+        self.assertEqual(USER_ROLE, self.app.store.get_user('user_role').role)
+
+    def test_add_user_with_invalid_role(self):
+        # Invalid roles
+        self._add_user("invalid", "invalid@test.com", "test2", "/home/", 'admin')
+        self.assertStatus(400)
+        self.assertInBody('role: Not a valid choice')
+
+        self._add_user("invalid", "invalid@test.com", "test2", "/home/", -1)
+        self.assertStatus(400)
+        self.assertInBody('role: Not a valid choice')
+
     def test_add_edit_delete(self):
         #  Add user to be listed
-        self._add_user("test2", "test2@test.com", "test2", "/home/", False)
-        try:
-            self.assertInBody("User added successfully.")
-            self.assertInBody("test2")
-            self.assertInBody("test2@test.com")
-            #  Update user
-            self._edit_user("test2", "chaned@test.com", "new-password", "/tmp/", True)
-            self.assertInBody("User information modified successfully.")
-            self.assertInBody("test2")
-            self.assertInBody("chaned@test.com")
-            self.assertNotInBody("/home/")
-            self.assertInBody("/tmp/")
-            #  Check with filters
-            self.getPage("/admin/users/?criteria=admins")
-            self.assertInBody("test2")
-        finally:
-            self._delete_user("test2")
-            self.assertInBody("User account removed.")
-            self.assertNotInBody("test2")
+        self._add_user("test2", "test2@test.com", "test2", "/home/", USER_ROLE)
+        self.assertInBody("User added successfully.")
+        self.assertInBody("test2")
+        self.assertInBody("test2@test.com")
+        #  Update user
+        self._edit_user("test2", "chaned@test.com", "new-password", "/tmp/", ADMIN_ROLE)
+        self.assertInBody("User information modified successfully.")
+        self.assertInBody("test2")
+        self.assertInBody("chaned@test.com")
+        self.assertNotInBody("/home/")
+        self.assertInBody("/tmp/")
+        #  Check with filters
+        self.getPage("/admin/users/?criteria=admins")
+        self.assertInBody("test2")
+
+        self._delete_user("test2")
+        self.assertInBody("User account removed.")
+        self.assertNotInBody("test2")
 
     def test_add_edit_delete_user_with_encoding(self):
         """
         Check creation of user with non-ascii char.
         """
-        self._add_user("Éric", "éric@test.com", "Éric", "/home/", False)
-        try:
-            self.assertInBody("User added successfully.")
-            self.assertInBody("Éric")
-            self.assertInBody("éric@test.com")
-            # Update user
-            self._edit_user("Éric", "eric.létourno@test.com", "écureuil", "/tmp/", True)
-            self.assertInBody("User information modified successfully.")
-            self.assertInBody("Éric")
-            self.assertInBody("eric.létourno@test.com")
-            self.assertNotInBody("/home/")
-            self.assertInBody("/tmp/")
-            # Check with filter
-            self.getPage("/admin/users/?criteria=admins")
-            self.assertInBody("Éric")
-        finally:
-            self._delete_user("Éric")
+        self._add_user("Éric", "éric@test.com", "Éric", "/home/", USER_ROLE)
+        self.assertInBody("User added successfully.")
+        self.assertInBody("Éric")
+        self.assertInBody("éric@test.com")
+        # Update user
+        self._edit_user("Éric", "eric.létourno@test.com", "écureuil", "/tmp/", ADMIN_ROLE)
+        self.assertInBody("User information modified successfully.")
+        self.assertInBody("Éric")
+        self.assertInBody("eric.létourno@test.com")
+        self.assertNotInBody("/home/")
+        self.assertInBody("/tmp/")
+        # Check with filter
+        self.getPage("/admin/users/?criteria=admins")
+        self.assertInBody("Éric")
+
+        self._delete_user("Éric")
         self.assertInBody("User account removed.")
         self.assertNotInBody("Éric")
 
@@ -125,15 +146,16 @@ class AdminUsersAsAdminTest(AbstractAdminTest):
         """
         Verify failure trying to create user without username.
         """
-        self._add_user("", "test1@test.com", "test1", "/tmp/", False)
-        self.assertInBody("The username is invalid.")
+        self._add_user("", "test1@test.com", "test1", "/tmp/", USER_ROLE)
+        self.assertStatus(400)
+        self.assertInBody("username: This field is required.")
 
     def test_add_user_with_existing_username(self):
         """
         Verify failure trying to add the same user.
         """
-        self._add_user("test1", "test1@test.com", "test1", "/tmp/", False)
-        self._add_user("test1", "test1@test.com", "test1", "/tmp/", False)
+        self._add_user("test1", "test1@test.com", "test1", "/tmp/", USER_ROLE)
+        self._add_user("test1", "test1@test.com", "test1", "/tmp/", USER_ROLE)
         self.assertInBody("User test1 already exists.")
 
     def test_add_user_with_invalid_root_directory(self):
@@ -144,20 +166,20 @@ class AdminUsersAsAdminTest(AbstractAdminTest):
             self._delete_user("test5")
         except:
             pass
-        self._add_user("test5", "test1@test.com", "test5", "/var/invalid/", False)
-        self.assertNotInBody("User added successfully.")
-        self.assertInBody("User root directory /var/invalid/ is not accessible!")
+        self._add_user("test5", "test1@test.com", "test5", "/var/invalid/", USER_ROLE)
+        self.assertInBody("User added successfully.")
+        self.assertInBody("User&#39;s root directory /var/invalid/ is not accessible!")
 
     def test_add_without_email(self):
         #  Add user to be listed
-        self._add_user("test2", None, "test2", "/tmp/", False)
+        self._add_user("test2", None, "test2", "/tmp/", USER_ROLE)
         self.assertInBody("User added successfully.")
 
     def test_add_without_user_root(self):
         #  Add user to be listed
-        self._add_user("test6", None, "test6", None, False)
+        self._add_user("test6", None, "test6", None, USER_ROLE)
         self.assertInBody("User added successfully.")
-        
+
         user = self.app.store.get_user('test6')
         self.assertEquals('', user.user_root)
 
@@ -175,13 +197,27 @@ class AdminUsersAsAdminTest(AbstractAdminTest):
         self._delete_user(self.USERNAME)
         self.assertInBody("You cannot remove your own account!")
 
+    def test_delete_user_admin(self):
+        """
+        Verify failure to delete our self.
+        """
+        # Create another admin user
+        self._add_user('admin2', '', 'password', '' , ADMIN_ROLE)
+        self.getPage("/logout/")
+        self._login('admin2', 'password')
+
+        # Try deleting admin user
+        self._delete_user(self.USERNAME)
+        self.assertStatus(200)
+        self.assertInBody("can&#39;t delete admin user")
+
     def test_edit_user_with_invalid_path(self):
         """
         Verify failure trying to update user with invalid path.
         """
-        self._edit_user("test1", "test1@test.com", "test", "/var/invalid/", False)
+        self._edit_user("test1", "test1@test.com", "test", "/var/invalid/", USER_ROLE)
         self.assertNotInBody("User added successfully.")
-        self.assertInBody("User root directory /var/invalid/ is not accessible!")
+        self.assertInBody("User&#39;s root directory /var/invalid/ is not accessible!")
 
     def test_list(self):
         self.getPage("/admin/users/")
@@ -193,7 +229,7 @@ class AdminUsersAsAdminTest(AbstractAdminTest):
         """
         Verify failure trying to update invalid user.
         """
-        self._edit_user("test4", "test1@test.com", "test1", "/tmp/", False)
+        self._edit_user("test4", "test1@test.com", "test1", "/tmp/", USER_ROLE)
         self.assertStatus(500)
 
     def test_criteria(self):
@@ -212,6 +248,23 @@ class AdminUsersAsAdminTest(AbstractAdminTest):
         self.getPage("/admin/users?search=coucou")
         self.assertNotInBody("test1")
 
+    def test_user_invalid_root(self):
+        # Delete all user's
+        for user in self.app.store.users():
+            if user.username != self.USERNAME:
+                user.delete()
+        # Change the user's root
+        user = self.app.store.get_user('admin')
+        user.user_root = "/invalid"
+        self.getPage("/admin/users")
+        self.assertInBody("Root directory not accessible!")
+
+        # Query the page by default
+        user = self.app.store.get_user('admin')
+        user.user_root = "/tmp/"
+        self.getPage("/admin/users")
+        self.assertNotInBody("Root directory not accessible!")
+
 
 class AdminUsersAsUserTest(AbstractAdminTest):
     """Integration test for page_admin"""
@@ -228,7 +281,7 @@ class AdminUsersAsUserTest(AbstractAdminTest):
         """
         Check if adding user is forbidden.
         """
-        self._add_user("test2", "test2@test.com", "test2", "/tmp/", False)
+        self._add_user("test2", "test2@test.com", "test2", "/tmp/", USER_ROLE)
         self.assertStatus(403)
 
     def test_delete_user(self):
@@ -242,7 +295,7 @@ class AdminUsersAsUserTest(AbstractAdminTest):
         """
         Check if editing user is forbidden.
         """
-        self._edit_user("test", "test1@test.com", "test", "/var/invalid/", False)
+        self._edit_user("test", "test1@test.com", "test", "/var/invalid/", USER_ROLE)
         self.assertStatus(403)
 
     def test_users(self):
@@ -251,7 +304,7 @@ class AdminUsersAsUserTest(AbstractAdminTest):
         """
         self.getPage("/admin/users")
         self.assertStatus(403)
-        
+
     def test_repos(self):
         """
         Check if listing user is forbidden.
@@ -267,7 +320,7 @@ class AdminLogsTest(WebCase):
     def test_no_logs(self):
         self.app.cfg['logfile'] = None
         self.app.cfg['logaccessfile'] = None
-        
+
         self.getPage("/admin/logs/")
         self.assertStatus(200)
         self.assertInBody("No log files")
@@ -275,31 +328,31 @@ class AdminLogsTest(WebCase):
     def test_logs(self):
         self.app.cfg['logfile'] = './rdiffweb.log'
         self.app.cfg['logaccessfile'] = './rdiffweb-access.log'
-        
+
         with open('./rdiffweb.log', 'w') as f:
             f.write("content of log file")
-        
+
         self.getPage("/admin/logs/")
         self.assertStatus(200)
         self.assertInBody("rdiffweb.log")
         self.assertInBody("content of log file")
         self.assertInBody("rdiffweb-access.log")
-        
+
         os.remove('./rdiffweb.log')
-        
+
     def test_logs_with_no_file(self):
         self.app.cfg['logfile'] = './rdiffweb.log'
         self.app.cfg['logaccessfile'] = './rdiffweb-access.log'
-        
+
         self.getPage("/admin/logs/")
         self.assertStatus(200)
         self.assertInBody("rdiffweb.log")
         self.assertInBody("Error getting file content")
-        
+
     def test_logs_with_invalid_file(self):
         self.app.cfg['logfile'] = './rdiffweb.log'
         self.app.cfg['logaccessfile'] = './rdiffweb-access.log'
-        
+
         self.getPage("/admin/logs/invalid")
         self.assertStatus(404)
 
@@ -307,27 +360,27 @@ class AdminLogsTest(WebCase):
 class AdminReposTest(WebCase):
 
     login = True
-    
+
     reset_app = True
-    
+
     reset_testcases = True
-    
+
     def test_repos(self):
         self.getPage("/admin/repos")
         self.assertStatus(200)
-        
+
     def test_repos_with_search(self):
         # Search something that exists
         self.getPage("/admin/repos?search=test")
         self.assertStatus(200)
         self.assertInBody(self.REPO)
-        
+
         # Search something that doesn't exists
         self.getPage("/admin/repos?search=coucou")
         self.assertStatus(200)
         self.assertNotInBody(self.REPO)
         self.assertInBody("No repository found")
-        
+
     def test_repos_with_criteria(self):
         # Search something that exists
         self.getPage("/admin/repos?criteria=ok")
@@ -339,12 +392,12 @@ class AdminReposTest(WebCase):
         self.assertStatus(200)
         self.assertNotInBody(self.REPO)
         self.assertInBody("No repository found")
-        
+
 
 class AdminSysinfoTest(WebCase):
 
     login = True
-    
+
     def test_sysinfo(self):
         self.getPage("/admin/sysinfo")
         self.assertStatus(200)
