@@ -15,6 +15,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 """
 Created on Jan 1, 2016
 
@@ -25,6 +26,7 @@ from __future__ import unicode_literals
 import logging
 import unittest
 
+from rdiffweb.core.store import USER_ROLE
 from rdiffweb.test import WebCase
 
 
@@ -96,9 +98,9 @@ class SettingsTest(WebCase):
         self.assertStatus('200 OK')
         self.assertInBody("Activities")
 
-        # Remove admin right
+        # Remove admin role
         admin = self.app.store.get_user('admin')
-        admin.is_admin = 0
+        admin.role = USER_ROLE
 
         # Browse admin's repos
         self.getPage("/graphs/activities/anotheruser/testcases")
