@@ -15,6 +15,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 """
 Created on Dec 29, 2015
 
@@ -26,6 +27,7 @@ from __future__ import unicode_literals
 import logging
 import unittest
 
+from rdiffweb.core.store import USER_ROLE
 from rdiffweb.test import WebCase
 
 
@@ -37,14 +39,14 @@ class HistoryPageTest(WebCase):
 
     reset_testcases = True
 
-    def _history(self, repo, limit=None):
-        url = "/history/" + repo + "/"
+    def _history(self, user, repo, limit=None):
+        url = "/history/" + user + "/" + repo + "/"
         if limit:
             url += "?limit=%s" % limit
         return self.getPage(url)
 
     def test_history(self):
-        self._history(self.REPO)
+        self._history(self.USERNAME, self.REPO)
         # New
         self.assertInBody("2016-02-02 16:30")
         # Old
@@ -52,24 +54,24 @@ class HistoryPageTest(WebCase):
         self.assertInBody("Show more")
 
     def test_history_with_limit(self):
-        self._history(self.REPO, 10)
+        self._history(self.USERNAME, self.REPO, 10)
         self.assertInBody("Show more")
-        self._history(self.REPO, 50)
+        self._history(self.USERNAME, self.REPO, 50)
         self.assertNotInBody("Show more")
 
     def test_as_another_user(self):
         # Create a nother user with admin right
         user_obj = self.app.store.add_user('anotheruser', 'password')
         user_obj.user_root = self.app.testcases
-        user_obj.repos = ['testcases']
-        
+        user_obj.add_repo('testcases')
+
         self.getPage("/history/anotheruser/testcases")
         self.assertStatus('200 OK')
-        
+
         # Remove admin right
         admin = self.app.store.get_user('admin')
-        admin.is_admin = 0
-        
+        admin.role = USER_ROLE
+
         # Browse admin's repos
         self.getPage("/history/anotheruser/testcases")
         self.assertStatus('403 Forbidden')
