@@ -15,6 +15,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 """
 Created on Apr 10, 2016
 
@@ -25,6 +26,7 @@ from __future__ import unicode_literals
 import logging
 import unittest
 
+from rdiffweb.core.store import MAINTAINER_ROLE, USER_ROLE
 from rdiffweb.test import WebCase
 
 
@@ -39,26 +41,28 @@ class DeleteRepoTest(WebCase):
     def _settings(self, repo):
         self.getPage("/settings/" + repo + "/")
 
-    def _delete(self, repo, confirm, **kwargs):
+    def _delete(self, user, repo, confirm, **kwargs):
         body = {}
         body.update({'action': 'delete'})
         if confirm is not None:
             body.update({'confirm': confirm})
         if kwargs:
             body.update(kwargs)
-        self.getPage("/settings/" + repo + "/", method="POST", body=body)
+        self.getPage("/settings/" + user + "/" + repo + "/", method="POST", body=body)
 
     def test_delete(self):
         """
         Check to delete a repo.
         """
-        self._delete(self.REPO, 'testcases')
+        self._delete(self.USERNAME, self.REPO, 'testcases')
         self.assertStatus(303)
         self.assertEqual([], self.app.store.get_user('admin').repos)
 
     def test_delete_with_slash(self):
-        self.app.store.get_user('admin').repos = ['/testcases']
-        self._delete(self.REPO, 'testcases')
+        # Make sure the repo exists.
+        self.assertEqual(['testcases'], self.app.store.get_user('admin').repos)
+        # Then delete it.
+        self._delete(self.USERNAME, self.REPO, 'testcases')
         self.assertStatus(303)
         self.assertEqual([], self.app.store.get_user('admin').repos)
 
@@ -66,7 +70,7 @@ class DeleteRepoTest(WebCase):
         """
         Check failure to delete a repo with wrong confirmation.
         """
-        self._delete(self.REPO, 'wrong')
+        self._delete(self.USERNAME, self.REPO, 'wrong')
         # TODO Make sure the repository is not delete
         self.assertStatus(400)
         self.assertEqual(['testcases'], self.app.store.get_user('admin').repos)
@@ -75,28 +79,61 @@ class DeleteRepoTest(WebCase):
         """
         Check failure to delete a repo with wrong confirmation.
         """
-        self._delete(self.REPO, None)
+        self._delete(self.USERNAME, self.REPO, None)
         # TODO Make sure the repository is not delete
         self.assertStatus(400)
         self.assertEqual(['testcases'], self.app.store.get_user('admin').repos)
-        
-    def test_as_another_user(self):
-        """
-        From admin user delete anotehr user repo.
-        """
-        # Create a nother user with admin right
+
+    def test_delete_as_admin(self):
+        # Create a another user with admin right
         user_obj = self.app.store.add_user('anotheruser', 'password')
         user_obj.user_root = self.app.testcases
-        user_obj.repos = ['testcases']
-        
-        self._delete('anotheruser/testcases', 'testcases', redirect='/admin/repos/')
+        user_obj.add_repo('testcases')
+
+        self._delete('anotheruser', 'testcases', 'testcases', redirect='/admin/repos/')
         self.assertStatus(303)
         location = self.assertHeader('Location')
         self.assertTrue(location.endswith('/admin/repos/'))
-        
+
         # Check database update
         self.assertEqual([], user_obj.repos)
 
+    def test_delete_as_maintainer(self):
+        # Create a another user with maintainer right
+        user_obj = self.app.store.add_user('maintainer', 'password')
+        user_obj.user_root = self.app.testcases
+        user_obj.add_repo('testcases')
+        user_obj.role = MAINTAINER_ROLE
+        
+        # Login as maintainer
+        self._login('maintainer', 'password')
+        
+        # Try to delete own own repo
+        self._delete('maintainer', 'testcases', 'testcases', redirect='/admin/repos/')
+        self.assertStatus(303)
+        location = self.assertHeader('Location')
+        self.assertTrue(location.endswith('/admin/repos/'))
+
+        # Check database update
+        self.assertEqual([], user_obj.repos)
+
+    def test_delete_as_user(self):
+        # Create a another user with maintainer right
+        user_obj = self.app.store.add_user('user', 'password')
+        user_obj.user_root = self.app.testcases
+        user_obj.add_repo('testcases')
+        user_obj.role = USER_ROLE
+
+        # Login as maintainer
+        self._login('user', 'password')
+        
+        # Try to delete own own repo
+        self._delete('user', 'testcases', 'testcases', redirect='/admin/repos/')
+        self.assertStatus(403)
+
+        # Check database update
+        self.assertEqual(['testcases'], user_obj.repos)
+
 
 if __name__ == "__main__":
     # import sys;sys.argv = ['', 'Test.testName']
